package org.lyj.node;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.knime.core.data.DataTableSpec;
import org.knime.core.node.InvalidSettingsException;
import org.knime.core.node.NodeDialogPane;
import org.knime.core.node.NodeSettingsRO;
import org.knime.core.node.NodeSettingsWO;
import org.knime.core.node.NotConfigurableException;
import org.knime.core.node.defaultnodesettings.DefaultNodeSettingsPane;
import org.knime.core.node.util.FilesHistoryPanel;
import org.knime.core.node.util.FilesHistoryPanel.LocationValidation;
import org.knime.core.node.workflow.FlowVariable;

/**
 * <code>NodeDialog</code> for the "ParametersInput" Node.
 * 参数输入节点
 *
 * This node dialog derives from {@link DefaultNodeSettingsPane} which allows
 * creation of a simple dialog with standard components. If you need a more 
 * complex dialog please derive directly from 
 * {@link org.knime.core.node.NodeDialogPane}.
 * 
 * @author YoGa
 */
public final class ParametersInputNodeDialog extends NodeDialogPane {

	private final JTextField m_sparkUrlField;
	
	/*
	 * 2017年12月21日15:52:33
	 * 删除
	 */
	//private final JTextField m_sparkMasterField;
	
	/*
	 * 2017年12月20日15:28:15
	 * 添加 username password
	 */
	private final JTextField m_usernameField;
	private final JTextField m_passwordField;
	
	private final JTextField m_executorCoresField;
	private final JTextField m_driverMemoryField;
	private final JTextField m_executorMemoryField;
	private final JTextField m_configSerializerField;
	private final JTextField m_configHeartbeatField;
	
	private final JTextField m_jarUrlField;
	
	/*
	 * 两个输入
	 * 2018年1月5日17:36:09
	 */
	
	private final JTextField m_InputRelationField;//输入--关系数据
	private final JTextField m_InputNodeField;//输入--节点数据
	private final JTextField m_InputOrgField;//输入--投资机构数据
	private final JTextField m_InputBfTimField;//输入--上次运行时间
	private final JTextField m_InputNowTimField;//输入--本次运行时间
	private final JTextField m_OutputRelationField;//输出--系信息
	private final JTextField m_OutputCompanyField;//输出--系成员信息
	private final JTextField m_OutputUBOField;//输出--UBO信息
	
	
	/*
	 * 
	 * 增加模型参数
	 * menCountMin.   capitalNum.  totalCapital.capitalRate.groupCapital.
	 * */
	private final JTextField m_MenCountMinField;//最小成员数---2
	private final JTextField m_CapitalNumField;//集团母公司对外投资数---5
	private final JTextField m_TotalCapitalField;//集团母公司认定合计注册资本---100000000
	private final JTextField m_CapitalRateField;//控制图，最小投资比例---10
	private final JTextField m_GroupCapitalField;//集团母公司注册资本最小值---500000000
	
	
	
	/*
	 * 2017年12月21日13:41:55
	 * 添加jar包选择按钮
	 */
	private final FilesHistoryPanel m_filePanel;
	
	/*
	 * 2018年1月5日10:30:17
	 * 添加class选择
	 */
	private final JTextField m_classField;
	
	public ParametersInputNodeDialog(){
		m_jarUrlField = new JTextField("",60);
		/*
		 * 2017年12月21日13:40:23
		 * 添加jar包选择按钮
		 */
		m_filePanel = new FilesHistoryPanel(createFlowVariableModel(ParametersConfig.CFG_URL,FlowVariable.Type.STRING), "jar_get", LocationValidation.FileInput, ".jar");
		m_filePanel.setDialogType(JFileChooser.OPEN_DIALOG);
		//m_filePanel.setToolTipText("浏览");
		m_filePanel.setShowConnectTimeoutField(false);
		
		/*
		 * 2017年12月21日15:52:50
		 * 删除
		 */
		//m_sparkMasterField = new JTextField("",60);
		m_sparkUrlField = new JTextField("###", 60);
		
		m_executorCoresField = new JTextField("",6);
		m_driverMemoryField =  new JTextField("",6);
		m_executorMemoryField = new JTextField("",6);
		m_configSerializerField = new JTextField("",20);
		m_configHeartbeatField = new JTextField("",5);
		
		m_InputRelationField = new JTextField("", 20);
		m_InputNodeField = new JTextField("", 20);
		m_InputOrgField = new JTextField("", 20);
		m_InputBfTimField = new JTextField("", 20);
		m_InputNowTimField = new JTextField("", 20);
		m_OutputRelationField = new JTextField("", 20);
		m_OutputCompanyField = new JTextField("", 20);
		m_OutputUBOField = new JTextField("", 20);
		
		/*
		 *模型参数设置 
		 * 
		 */
		m_MenCountMinField = new JTextField("",15);
		m_CapitalNumField = new JTextField("",15);
		m_TotalCapitalField = new JTextField("",15);
		m_CapitalRateField = new JTextField("",15);
		m_GroupCapitalField = new JTextField("",15);
		
		/*
		 * 2017年12月20日15:27:24
		 * 添加 username password
		 */
		m_usernameField = new JTextField("", 20);
		m_passwordField = new JTextField("", 20);
		
		/*
		 * 2018年1月5日10:31:04
		 * 添加class选择
		 */
		m_classField = new JTextField("", 20);
		
		/*
		 * 2018年1月5日17:36:58
		 * 第二个输入
		 */
		
		addTab("运行参数", initRunLayout());
		addTab("模型参数", initModeLayout());
	}
	
	private JPanel initRunLayout(){
		/*JPanel all_panel = new JPanel();*/
		
		/*
		 * 2017年12月21日13:47:20
		 * 添加jar包选取功能
		 */
		final JPanel jarChosPanel = new JPanel();
		jarChosPanel.setLayout(new BoxLayout(jarChosPanel, BoxLayout.X_AXIS));
		jarChosPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"JAR选取："));
		jarChosPanel.setMaximumSize(new Dimension(Integer.MAX_VALUE,m_filePanel.getPreferredSize().height));
		jarChosPanel.add(m_filePanel);

		
		/*Spark服务器URL*/
		final JPanel sparkurlPanel = new JPanel();
		sparkurlPanel.setLayout(new BoxLayout(sparkurlPanel, BoxLayout.X_AXIS));
		sparkurlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"Spark服务器地址:"));
		sparkurlPanel.setMaximumSize(new Dimension(Integer.MAX_VALUE,m_sparkUrlField.getPreferredScrollableViewportSize().height));

		sparkurlPanel.add(getInFlowLayout(m_sparkUrlField),new JLabel("url"));
		
		
		/*
		 * 2017年12月20日15:26:46
		 * 添加username password
		 */
		JPanel uspaPanel = new JPanel(new GridBagLayout());
		uspaPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),"登录用户名及密码:"));
		GridBagConstraints gbc_uspa = new GridBagConstraints();
		gbc_uspa.gridx = 0;
		gbc_uspa.gridy = 0;
		gbc_uspa.insets = new Insets(5, 5, 5, 5);
		gbc_uspa.anchor = GridBagConstraints.FIRST_LINE_START;
		uspaPanel.add(getInFlowLayout(new JLabel("用户名："),m_usernameField),gbc_uspa);
		gbc_uspa.gridx += 1;
		uspaPanel.add(getInFlowLayout(new JLabel("密码："),m_passwordField),gbc_uspa);
		
		/*机器配置*/
		JPanel executorPanel = new JPanel(new GridBagLayout());
		executorPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
	            .createEtchedBorder(), "机器配置："));
	    GridBagConstraints gbc_1 = new GridBagConstraints();
	    gbc_1.gridx = 0;
	    gbc_1.gridy = 0;
	    gbc_1.insets = new Insets(5, 5, 5, 5);
	    gbc_1.anchor = GridBagConstraints.FIRST_LINE_START;
		
		executorPanel.add(getInFlowLayout(new JLabel("执行cpu核数："),m_executorCoresField),gbc_1);
		gbc_1.gridx += 1;
//		optionsPanel.add(new JLabel("Executor内存大小:"));
		executorPanel.add(getInFlowLayout(new JLabel("Executor内存大小："),m_executorMemoryField,new JLabel("G")),gbc_1);
		
		gbc_1.gridx += 1;
//		optionsPanel.add(new JLabel("Driver使用内存大小:"));
		executorPanel.add(getInFlowLayout(new JLabel("Driver内存大小："),m_driverMemoryField,new JLabel("G")),gbc_1);
		/*gbc_1.gridx = 0;
        gbc_1.gridy += 1;
        gbc_1.weightx = 0;*/
        
        /*可选配置*/
        JPanel optionsPanel = new JPanel(new GridBagLayout());
		optionsPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
	            .createEtchedBorder(), "可选配置："));
		GridBagConstraints gbc_2 = new GridBagConstraints();
	    gbc_2.gridx = 0;
	    gbc_2.gridy = 0;
	    gbc_2.insets = new Insets(5, 5, 5, 5);
	    gbc_2.anchor = GridBagConstraints.FIRST_LINE_START;
	    optionsPanel.add(getInFlowLayout(new JLabel("序列化方式（可为空）："),m_configSerializerField),gbc_2);
	    
	    gbc_2.gridx+=1;
	    optionsPanel.add(getInFlowLayout(new JLabel("心跳时长（可为空）："),m_configHeartbeatField,new JLabel("秒")),gbc_2);
	    
	    /*输入输出*/
	    JPanel inOutPanel = new JPanel(new GridBagLayout());
	    inOutPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
	            .createEtchedBorder(), "输入输出数据配置:"));
		GridBagConstraints gbc_3 = new GridBagConstraints();
		gbc_3.gridx = 0;
		gbc_3.gridy = 0;
		gbc_3.insets = new Insets(5, 5, 5, 5);
		gbc_3.anchor = GridBagConstraints.FIRST_LINE_START;
		
		/*
		 * 2018年1月5日17:38:11
		 * 加入第二个输入
		 */
		inOutPanel.add(getInFlowLayout(new JLabel("关系数据表(输入)："),m_InputRelationField),gbc_3);
	    gbc_3.gridx += 1;
//	    gbc_3.gridy += 1;
//	    gbc_3.weightx = 0;
		
		inOutPanel.add(getInFlowLayout(new JLabel("节点数据表(输入)："),m_InputNodeField),gbc_3);
		gbc_3.gridx += 1;
//	    gbc_3.gridy += 1;
//	    gbc_3.weightx = 0;
	    
	    inOutPanel.add(getInFlowLayout(new JLabel("投资机构表(输入)："),m_InputOrgField),gbc_3);
	    gbc_3.gridx -= 2;
	    gbc_3.gridy += 1;
//	    gbc_3.weightx = 0;
	    
	    inOutPanel.add(getInFlowLayout(new JLabel("上次运行日期："),m_InputBfTimField),gbc_3);
	    gbc_3.gridx += 1;
	    
	    inOutPanel.add(getInFlowLayout(new JLabel("本次运行日期："),m_InputNowTimField),gbc_3);
	    gbc_3.gridx -= 1;
	    gbc_3.gridy += 1;
//	    gbc_3.weightx = 0;
	    
	    inOutPanel.add(getInFlowLayout(new JLabel("系信息表（输出）："),m_OutputRelationField),gbc_3);
	    gbc_3.gridx += 1;
//	    gbc_3.gridy = 0;
//	    gbc_3.weightx = 0;
	    inOutPanel.add(getInFlowLayout(new JLabel("系成员表（输出）："),m_OutputCompanyField),gbc_3);
	    gbc_3.gridx += 1;
//	    gbc_3.gridy += 1;
//	    gbc_3.weightx = 0;
	    inOutPanel.add(getInFlowLayout(new JLabel("强经济圈成员表（输出）："),m_OutputUBOField),gbc_3);
        
	    
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.add(sparkurlPanel);
		panel.add(Box.createVerticalStrut(2));
		/*
		 * 2017年12月20日15:38:09
		 * 添加username password
		 */
		panel.add(uspaPanel);
		panel.add(Box.createHorizontalStrut(2));
		
		
		panel.add(executorPanel);
		panel.add(Box.createVerticalStrut(2));
//		隐藏可选参数输入
//		panel.add(optionsPanel);
		panel.add(Box.createVerticalStrut(2));
		panel.add(inOutPanel);
		
		panel.add(Box.createVerticalStrut(2));
		panel.add(jarChosPanel);
		/*all_panel.add(panel);
		all_panel.add(executorPanel);*/
		
		return panel;
	}

	private JPanel initModeLayout(){
		
		/*模型参数*/
	    JPanel modePanel = new JPanel(new GridBagLayout());
	    modePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
	            .createEtchedBorder(), "模型参数:"));
		GridBagConstraints gbc_4 = new GridBagConstraints();
		gbc_4.gridx = 0;
		gbc_4.gridy = 0;
		gbc_4.insets = new Insets(5, 5, 5, 5);
		gbc_4.anchor = GridBagConstraints.FIRST_LINE_START;
		
		
		/*
		 * 2018年1月5日17:38:11
		 * 加入第二个输入
		 */
		modePanel.add(getInFlowLayout(new JLabel("最小成员数："),m_MenCountMinField),gbc_4);
	    gbc_4.gridy += 1;
	    gbc_4.gridx = 0;
	    gbc_4.weightx = 0;
		
	    modePanel.add(getInFlowLayout(new JLabel("集团母公司对外投资数："),m_CapitalNumField),gbc_4);
	    gbc_4.gridy += 1;
	    gbc_4.gridx = 0;
	    gbc_4.weightx = 0;
	    
		modePanel.add(getInFlowLayout(new JLabel("集团母公司认定合计注册资本："),m_TotalCapitalField),gbc_4);
	    gbc_4.gridy += 1;
	    gbc_4.gridx = 0;
	    gbc_4.weightx = 0;
	    
	    modePanel.add(getInFlowLayout(new JLabel("控制图，最小投资比例："),m_CapitalRateField),gbc_4);
	    gbc_4.gridy += 1;
	    gbc_4.gridx = 0;
	    gbc_4.weightx = 0;
	    
	    modePanel.add(getInFlowLayout(new JLabel("集团母公司注册资本最小值："),m_GroupCapitalField),gbc_4);

		
		JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.add(modePanel);
		panel.add(Box.createVerticalStrut(2));
		
		
		return panel;
		
	}
	private static JPanel getInFlowLayout(final JComponent... component){
		JPanel p = new JPanel(new FlowLayout(FlowLayout.LEFT));
		for(JComponent c : component){
			p.add(c);
		}
		return p;
	}
	
	
    @Override
	protected void loadSettingsFrom(NodeSettingsRO settings, DataTableSpec[] specs) throws NotConfigurableException {
//		super.loadSettingsFrom(settings, specs);
    	
    	ParametersConfig config = new ParametersConfig();
    	config.loadSettingInDialog(settings);
    	m_sparkUrlField.setText(escape(config.getM_sparkUrl()));
    	m_executorCoresField.setText(escape(config.getM_executorCores()));
    	m_executorMemoryField.setText(escape(config.getM_executorMemory()));
    	m_driverMemoryField.setText(escape(config.getM_driverMemory()));
    	m_configSerializerField.setText(config.getM_config_serializer());
    	m_configHeartbeatField.setText(config.getM_config_Heartbeat());
    	
    	m_jarUrlField.setText(config.getM_jarUrl());
    	
    	/*
    	 * 2017年12月20日15:39:07
    	 * 添加 username password
    	 */
    	m_usernameField.setText(config.getM_username());
    	m_passwordField.setText(config.getM_password());
    	
    	m_InputRelationField.setText(config.getM_inputRelation());
		m_InputNodeField.setText(config.getM_inputNode());
		m_InputOrgField.setText(config.getM_inputOrg());
		m_InputBfTimField.setText(config.getM_inputBfTim());
		m_InputNowTimField.setText(config.getM_inputNowTim());
		m_OutputRelationField.setText(config.getM_outputRelation());
		m_OutputCompanyField.setText(config.getM_outputCompany());
		m_OutputUBOField.setText(config.getM_OutputUBO());
		m_MenCountMinField.setText(config.getM_menCountMin());
		m_CapitalNumField.setText(config.getM_capitalNum());
		m_TotalCapitalField.setText(config.getM_totalCapital());
		m_CapitalRateField.setText(config.getM_capitalRate());
		m_GroupCapitalField.setText(config.getM_groupCapital());
    	
    	/*
    	 * 2017年12月21日13:54:02
    	 * 添加jar包选择
    	 */
    	m_filePanel.updateHistory();
    	m_filePanel.setSelectedFile(config.getM_jarUrl());
    	m_filePanel.setConnectTimeout(config.getM_connectTimeout());
    	
    	/*
    	 * 2018年1月5日10:34:37
    	 * 添加class输入
    	 */
    	m_classField.setText(config.getM_class());
    	
    	
	}

	
	
	@Override
	protected void saveSettingsTo(final NodeSettingsWO settings) throws InvalidSettingsException {
		// TODO 将config写入传递
		System.out.println("---test：sparkUrl is ‘"+m_sparkUrlField.getText()+"’");
		ParametersConfig config = new ParametersConfig();
		
		config.setM_sparkUrl(m_sparkUrlField.getText());
		
		config.setM_executorCores(m_executorCoresField.getText());
		config.setM_executorMemory(m_executorMemoryField.getText());
		config.setM_driverMemory(m_driverMemoryField.getText());
		
		config.setM_config_Heartbeat(m_configHeartbeatField.getText());
		config.setM_config_serializer(m_configSerializerField.getText());
		config.setM_class(m_classField.getText());// TODO jar包用到的类
		//config.setM_class(m_classField.getText());
		
		config.setM_name("Test_"+new SimpleDateFormat ("yyyyMMddhhmmss").format(new Date()));
		
		config.setM_inputRelation(m_InputRelationField.getText());
		config.setM_inputNode(m_InputNodeField.getText());
		config.setM_inputBfTim(m_InputBfTimField.getText());
		config.setM_inputOrg(m_InputOrgField.getText());
		config.setM_inputNowTim(m_InputNowTimField.getText());
		
		config.setM_outputRelation(m_OutputRelationField.getText());
		config.setM_outputCompany(m_OutputCompanyField.getText());
		config.setM_OutputUBO(m_OutputUBOField.getText());
		
		config.setM_menCountMin(m_MenCountMinField.getText());
		config.setM_capitalNum(m_CapitalNumField.getText());
		config.setM_totalCapital(m_TotalCapitalField.getText());
		config.setM_capitalRate(m_CapitalRateField.getText());
		config.setM_groupCapital(m_GroupCapitalField.getText());
		
    	
		
		/*
		 * 2017年12月20日15:40:26
		 * 添加username password
		 */
		config.setM_username(m_usernameField.getText());
		config.setM_password(m_passwordField.getText());
		
		/*
		 * 2017年12月21日14:08:26
		 * 添加Jar选取
		 */
		String fileL = m_filePanel.getSelectedFile().trim();
		config.setM_jarUrl(fileL);
		config.setM_connectTimeout(m_filePanel.getConnectTimeout().orElse(null));
		
		config.saveSettingsTo(settings);
		
		/*
		 * 2017年12月21日14:10:19
		 * jar选取添加到历史
		 */
		m_filePanel.addToHistory();
		
	}
	
	@SuppressWarnings("unused")
	private static String unescape(final String s) {
        if ("\\t".equals(s)) {
            return "\t";
        } else if ("\\n".equals(s)) {
            return "\n";
        } else if ("\\r".equals(s)) {
            return "\r";
        } else if ("\\r\\n".equals(s)) {
            return "\r\n";
        } else {
            return s;
        }
    }
	
	private static String escape(final String s){
		if ("\t".equals(s)) {
            return "\\t";
        } else if ("\n".equals(s)) {
            return "\\n";
        } else if ("\r".equals(s)) {
            return "\\r";
        } else if ("\r\n".equals(s)) {
            return "\\r\\n";
        } else {
            return s;
        }
	}
	
}

