package org.lyj.node;

import java.io.File;
import java.io.IOException;

import org.knime.core.data.DataCell;
import org.knime.core.data.DataColumnSpec;
import org.knime.core.data.DataColumnSpecCreator;
import org.knime.core.data.DataRow;
import org.knime.core.data.DataTableSpec;
import org.knime.core.data.RowKey;
import org.knime.core.data.def.DefaultRow;
import org.knime.core.data.def.StringCell;
import org.knime.core.node.BufferedDataContainer;
import org.knime.core.node.BufferedDataTable;
import org.knime.core.node.CanceledExecutionException;
import org.knime.core.node.ExecutionContext;
import org.knime.core.node.ExecutionMonitor;
import org.knime.core.node.InvalidSettingsException;
import org.knime.core.node.NodeCreationContext;
import org.knime.core.node.NodeLogger;
import org.knime.core.node.NodeModel;
import org.knime.core.node.NodeSettingsRO;
import org.knime.core.node.NodeSettingsWO;


/**
 * This is the model implementation of ParametersInput.
 * 参数输入节点
 *
 * @author YoGa
 */
public class ParametersInputNodeModel extends NodeModel {
    
	private ParametersConfig m_config;
	
	 /**
     * Constructor for the node model.
     */
    protected ParametersInputNodeModel() {
        super(0, 1);
    }
	
    ParametersInputNodeModel(final NodeCreationContext context){
    	this();
    	m_config = new ParametersConfig();
//    	m_config.setM_class("com.knowlegene.kw.deptgroup.One");
//    	m_config.setM_class("org.lyj.ke.test.Test");
//    	m_config.setM_name("One_"+ new SimpleDateFormat ("yyyy-MM-dd hh:mm:ss").format(new Date()));
    	m_config.setM_sparkUrl(context.getUrl().toString());
//    	m_config.setM_executorCores(context.get);
    	
    }
    
    protected ParametersConfig getConfig(){
    	return m_config;
    }
    
    
    
    // the logger instance
    private static final NodeLogger logger = NodeLogger
            .getLogger(ParametersInputNodeModel.class);
        
    /** the settings key which is used to retrieve and 
        store the settings (from the dialog or from a settings file)    
       (package visibility to be usable from the dialog). */
//	static final String CFGKEY_COUNT = "Count";

    /** initial default count value. */
//    static final int DEFAULT_COUNT = 100;

    // example value: the models count variable filled from the dialog 
    // and used in the models execution method. The default components of the
    // dialog work with "SettingsModels".
    /*private final SettingsModelIntegerBounded m_count =
        new SettingsModelIntegerBounded(ParametersInputNodeModel.CFGKEY_COUNT,
                    ParametersInputNodeModel.DEFAULT_COUNT,
                    Integer.MIN_VALUE, Integer.MAX_VALUE);*/
    

   

    /**
     * {@inheritDoc}
     */
    @Override
    protected BufferedDataTable[] execute(final BufferedDataTable[] inData,
            final ExecutionContext exec) throws Exception {
    	System.out.println("-----START-----");
    	logger.info("-----START-----");
//    	System.out.println("------------exec:"+exec.toString());
    	System.out.println("------------");
    	/*System.out.println("输入的内容为：");
    	System.out.println("url path=== " + m_config.getM_sparkUrl());
    	System.out.println("jar path=== " + m_config.getM_jarUrl());
    	System.out.println("class   === " + m_config.getM_class());
    	System.out.println("name    === " + m_config.getM_name());
    	System.out.println("------------");*/
        logger.info("生成配置信息table...");
    	//System.out.println("-----create settings table ...");
        //System.out.println("-----Creating 12 column...");
        // the data table spec of the single output table, 
        // the table will have three columns:
        DataColumnSpec[] allColSpecs = new DataColumnSpec[24];
        System.out.println("-----24 columns ");
        allColSpecs[0] = new DataColumnSpecCreator("Class", StringCell.TYPE).createSpec();//
        allColSpecs[1] = new DataColumnSpecCreator("SparkUrl", StringCell.TYPE).createSpec();
        allColSpecs[2] = new DataColumnSpecCreator("CName", StringCell.TYPE).createSpec();
        allColSpecs[3] = new DataColumnSpecCreator("ExecutorCores", StringCell.TYPE).createSpec();
        allColSpecs[4] = new DataColumnSpecCreator("ExecutorMemory", StringCell.TYPE).createSpec();
        allColSpecs[5] = new DataColumnSpecCreator("DirverMemory", StringCell.TYPE).createSpec();
        allColSpecs[6] = new DataColumnSpecCreator("Serializer", StringCell.TYPE).createSpec();
        allColSpecs[7] = new DataColumnSpecCreator("Heartbeat", StringCell.TYPE).createSpec();
        allColSpecs[8] = new DataColumnSpecCreator("JarUrl", StringCell.TYPE).createSpec();
        allColSpecs[9] = new DataColumnSpecCreator("UserName", StringCell.TYPE).createSpec();
        allColSpecs[10] = new DataColumnSpecCreator("PassWord", StringCell.TYPE).createSpec();
        allColSpecs[11] = new DataColumnSpecCreator("InputRelation", StringCell.TYPE).createSpec();
        allColSpecs[12] = new DataColumnSpecCreator("InputNode", StringCell.TYPE).createSpec();
        allColSpecs[13] = new DataColumnSpecCreator("InputOrg", StringCell.TYPE).createSpec();
        allColSpecs[14] = new DataColumnSpecCreator("InputBfTim", StringCell.TYPE).createSpec();
        allColSpecs[15] = new DataColumnSpecCreator("InputNowTim", StringCell.TYPE).createSpec();
        allColSpecs[16] = new DataColumnSpecCreator("OutputRelation", StringCell.TYPE).createSpec();
        allColSpecs[17] = new DataColumnSpecCreator("OutputCompany", StringCell.TYPE).createSpec();
        allColSpecs[18] = new DataColumnSpecCreator("OutputUBO", StringCell.TYPE).createSpec();
        allColSpecs[19] = new DataColumnSpecCreator("MemCountMin", StringCell.TYPE).createSpec();
        allColSpecs[20] = new DataColumnSpecCreator("CapitalNum", StringCell.TYPE).createSpec();
        allColSpecs[21] = new DataColumnSpecCreator("TotalCapital", StringCell.TYPE).createSpec();
        allColSpecs[22] = new DataColumnSpecCreator("CapitalRate", StringCell.TYPE).createSpec();
        allColSpecs[23] = new DataColumnSpecCreator("GroupCapital", StringCell.TYPE).createSpec();
        
       /* allColSpecs[1] = 
            new DataColumnSpecCreator("Column 1", DoubleCell.TYPE).createSpec();
        allColSpecs[2] = 
            new DataColumnSpecCreator("Column 2", IntCell.TYPE).createSpec();*/
        System.out.println("-----Creating One Table...");
        DataTableSpec outputSpec = new DataTableSpec(allColSpecs);
        // the execution context will provide us with storage capacity, in this
        // case a data container to which we will add rows sequentially
        // Note, this container can also handle arbitrary big data tables, it
        // will buffer to disc if necessary.
        System.out.println("-----Creating BufferedDataContainer...");
        BufferedDataContainer container = exec.createDataContainer(outputSpec);
        // let's add m_count rows to it
//        for (int i = 0; i < 50; i++) {
        System.out.println("-----Creating Rowkey Row1...");
        RowKey key_01 = new RowKey("Data " + 1);
//        RowKey key_02 = new RowKey("Row " + 2);
            // the cells of the current row, the types of the cells must match
            // the column spec (see above)
        System.out.println("-----Creating 24 DataCell...");
        DataCell[] cells = new DataCell[24];
        
        System.out.println("-----Adding data to DataCell...");
        
        cells[0] = new StringCell(m_config.getM_class());
        cells[1] = new StringCell(m_config.getM_sparkUrl());
        cells[2] = new StringCell(m_config.getM_name());
        cells[3] = new StringCell(m_config.getM_executorCores());
        cells[4] = new StringCell(m_config.getM_executorMemory());
        cells[5] = new StringCell(m_config.getM_driverMemory());
        cells[6] = new StringCell(m_config.getM_config_serializer());
        cells[7] = new StringCell(m_config.getM_config_Heartbeat());
        cells[8] = new StringCell(m_config.getM_jarUrl());
        cells[9] = new StringCell(m_config.getM_username());
        cells[10] = new StringCell(m_config.getM_password());
        cells[11] = new StringCell(m_config.getM_inputRelation());
        cells[12] = new StringCell(m_config.getM_inputNode());
        cells[13] = new StringCell(m_config.getM_inputOrg());
        cells[14] = new StringCell(m_config.getM_inputBfTim());
        cells[15] = new StringCell(m_config.getM_inputNowTim());
        cells[16] = new StringCell(m_config.getM_outputRelation());
        cells[17] = new StringCell(m_config.getM_outputCompany());
        cells[18] = new StringCell(m_config.getM_OutputUBO());
        cells[19] = new StringCell(m_config.getM_menCountMin());
        cells[20] = new StringCell(m_config.getM_capitalNum());
        cells[21] = new StringCell(m_config.getM_totalCapital());
        cells[22] = new StringCell(m_config.getM_capitalRate());
        cells[23] = new StringCell(m_config.getM_groupCapital());
        
        /*
         * TODO 添加第二个输入
         * 2018年1月5日17:48:46
         */
//        cells[14] = new StringCell(m_config.getM_input());
        
        System.out.println("-----Adding cell and key to DataRow...");
        DataRow row = new DefaultRow(key_01, cells);
        
        System.out.println("-----Adding row to table");
        container.addRowToTable(row);
            
            // check if the execution monitor was canceled
           /* exec.checkCanceled();
            exec.setProgress(i / (double)50, 
                "Adding row " + i);*/
//        }
        // once we are done, we close the container and return its table
        
        System.out.println("-----closing container...");
        container.close();
        
        System.out.println("-----Adding container to BufferedDataTable...");
        logger.info("-----Adding container to BufferedDataTable...");
        BufferedDataTable out = container.getTable();
        
        System.out.println("-----FINISHED-----");
        logger.info("-----FINISHED-----");
        return new BufferedDataTable[]{out};
    }

   /* protected FileTable createFileTable(final ExecutionContext exec){
    	
    	final DataTableSpec tableSpec = setting
    	
    	return new FileTable(tableSpec, settings, exec);
    	
    	
    }*/
    
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void reset() {
        // Models build during execute are cleared here.
        // Also data handled in load/saveInternals will be erased here.
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected DataTableSpec[] configure(final DataTableSpec[] inSpecs)
            throws InvalidSettingsException {
        
        // table structure, and the incoming types are feasible for the node
        // to execute. If the node can execute in its current state return
        // the spec of its output data table(s) (if you can, otherwise an array
        // with null elements), or throw an exception with a useful user message
    	
        return new DataTableSpec[]{null};
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void saveSettingsTo(final NodeSettingsWO settings) {

        // TODO save user settings to the config object.
       /* if (m_config != null) {
        	m_count.saveSettingsTo(settings);
		}*/
    	if (m_config != null) {
    		m_config.saveSettingsTo(settings);
		}
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void loadValidatedSettingsFrom(final NodeSettingsRO settings)
            throws InvalidSettingsException {
            
        // TODO load (valid) settings from the config object.
        // It can be safely assumed that the settings are valided by the 
        // method below.
        ParametersConfig config = new ParametersConfig();
//        m_count.loadSettingsFrom(settings);
        config.loadSettingInModel(settings);
        m_config = config;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected void validateSettings(final NodeSettingsRO settings)
            throws InvalidSettingsException {
            
        // TODO check if the settings could be applied to our model
        // e.g. if the count is in a certain range (which is ensured by the
        // SettingsModel).
        // Do not actually set any values of any member variables.
       // m_count.validateSettings(settings);
    	new ParametersConfig().loadSettingInModel(settings);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void loadInternals(final File internDir,
            final ExecutionMonitor exec) throws IOException,
            CanceledExecutionException {
        
        // Everything handed to output ports is loaded automatically (data
        // returned by the execute method, models loaded in loadModelContent,
        // and user settings set through loadSettingsFrom - is all taken care 
        // of). Load here only the other internals that need to be restored
        // (e.g. data used by the views).

    	
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void saveInternals(final File internDir,
            final ExecutionMonitor exec) throws IOException,
            CanceledExecutionException {
       
        // Everything written to output ports is saved automatically (data
        // returned by the execute method, models saved in the saveModelContent,
        // and user settings saved through saveSettingsTo - is all taken care 
        // of). Save here only the other internals that need to be preserved
        // (e.g. data used by the views).

    }

}

